#include "./CoreComponents.hpp"

namespace sva {
	/*
	 * GuiMovableWindow
	 */
	GuiMovableWindow::GuiMovableWindow() : m_WndRect({ 20,20, 200, 100 })
	{
		m_Title = "Example movable Window";
	}

	int GuiMovableWindow::input()
	{
		if (!m_WindowOpen) return 0;
		Vector2 mouse_pos = GetMousePosition();

		if (IsMouseButtonPressed(MOUSE_BUTTON_LEFT) && !m_DragWindow)
		{
			if (CheckCollisionPointRec(mouse_pos, { m_WndRect.x, m_WndRect.y, m_WndRect.width - 24, 20 }))
			{
				m_DragWindow = true;
				m_PanOffset = { mouse_pos.x - m_WndRect.x, mouse_pos.y - m_WndRect.y };
			}
		}

		if (m_DragWindow)
		{
			m_WndRect.x = (mouse_pos.x - m_PanOffset.x);
			m_WndRect.y = (mouse_pos.y - m_PanOffset.y);
			if (IsMouseButtonReleased(MOUSE_BUTTON_LEFT)) m_DragWindow = false;
		}
		return rinput(mouse_pos);
	}

	int GuiMovableWindow::rinput(Vector2& mouse_position)
	{
		return 0;
	}

	int GuiMovableWindow::rdraw()
	{
		return 0;
	}

	int GuiMovableWindow::draw()
	{
		if (!m_WindowOpen) return 0;

		m_WindowOpen = !GuiWindowBox(m_WndRect, m_Title.c_str());

		return rdraw();
	}

	void GuiMovableWindow::scale(float scale)
	{
		GuiComponent::scale(scale);
		m_WndRect.width *= scale;
		m_WndRect.height *= scale;
	}

	/*
	 * Error Window
	 */

	void ErrorWindow::onAttach()
	{
		m_WindowOpen = false;
	}

	int ErrorWindow::rdraw()
	{
		GuiLabel({ m_WndRect.x + m_TextAnchor.x, m_WndRect.y + m_TextAnchor.y, m_WndRect.width, m_WndRect.height }, m_Msg.c_str());

		return 0;
	}

	void ErrorWindow::onResize()
	{
		m_WndRect.x = (static_cast<float>(m_WndWidth) - m_WndRect.width) / 2;
		m_WndRect.y = (static_cast<float>(m_WndHeight) - m_WndRect.height) / 2;
	}

	void ErrorWindow::error(std::string title, std::string msg)
	{
		m_Msg = std::move(msg);
		float text_padding = 20;
		m_WndRect.width = static_cast<float>(MeasureText(m_Msg.c_str(), GuiGetStyle(DEFAULT, TEXT_SIZE))) + text_padding;
		m_WndRect.height = (m_WndRect.width * 2) / 3;

		m_TextAnchor = {
			.x = text_padding / 2,
			.y = m_WndRect.height / 2 - static_cast<float>(GuiGetStyle(DEFAULT, TEXT_SIZE))
		};

		m_WindowOpen = true;
		m_Title = std::move(title);
	}
}
